// PWMDualTest.c
// Runs on TM4C123

// Please note:
/*
   This is a modified version from original Jonathan Valvano's routine
	 
	 This is an example routine I prepared to handle a hobby servo motor 
	 with a potentiometer position
	 by PWM tested with a EK-TM4C123GXL board TM4C123.
	 Connections are:
	 PB6 is for PWM input on servo motor
	 Tested with a standard Tower Pro MG90S Micro servo motor
	 where 20 milliseconds is the standard period for the PWM input
	 0.9 ms is for +90 position
	 1.65 ms is for 0 position
	 2.4 ms is for -90  position
	 by Francisco Dominguez Roman 2014 Twitter: @Franciscodr
	 blog http://www.gestionconsult.es/blog/
	 more information of robotics at http://letsmakerobots.com/content/lets-learn-basics-robotics-edxs-embedded-systems-shape-world
	 and http://e2e.ti.com/support/microcontrollers/tiva_arm/f/908/p/342633/1201161.aspx

*/ 

// Old documentation from original version of this routine before adaptation to servo control

// Use PWM0/PB6 and PWM1/PB7 to generate two square waves with 50% duty cycle
// and 180 degrees out of phase.  These two signals plus two digital logic
// inverters are used to drive a stepper motor.  The stepper motor has 200
// steps per revolution and needs to be driven between 1 and 100 revolutions
// per minute.
// 1 rev   1 min    200 step   0.25 pulse   0.83333 pulse
// ----- * ------ * -------- * ---------- = -------------
// 1 min   60 sec    1 rev       1 step         1 sec    
// Daniel Valvano
// September 10, 2013

/* This example accompanies the book
   "Embedded Systems: Real Time Interfacing to ARM Cortex M Microcontrollers",
   ISBN: 978-1463590154, Jonathan Valvano, copyright (c) 2013
  Program 6.7, section 6.3.2

 Copyright 2013 by Jonathan W. Valvano, valvano@mail.utexas.edu
    You may use, edit, run or distribute this file
    as long as the above copyright notice remains
 THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 For more information about my classes, my research, and my books, see
 http://users.ece.utexas.edu/~valvano/
 */
 
#include "ADC.h"
//#include "UART.h" needed function included in this file to avoid a "Error: L6200E: Symbol multiply defined"
#include "PLL.h"
#include "PWMDual.h"
#include "PWMDual1.h"
#include "tm4c123gh6pm.h"
#include "Nokia5110.h"
#include "TExaS.h"

void EnableInterrupts(void);  // Enable interrupts

void UART_ConvertDistance(unsigned long n);
unsigned long LeerDato(void); //Noise reduction function
unsigned long ADC0_In(void); // ADC in function
unsigned char String[10];
unsigned long Distance; // units 0.001 cm
unsigned long ADCdata;  // 12-bit 0 to 4095 sample
unsigned long Flag;     // 1 means valid Distance, 0 means Distance is empty

void WaitForInterrupt(void);  // low power mode

// Initialize Systick for ADC
// Initialize SysTick interrupts to trigger at 40 Hz, 25 ms
// period = 2000000 to have interrupts at 40 Hz; period = 16000 => 1ms
void SysTick_Init(unsigned long period){
  volatile unsigned long delay;

	NVIC_ST_CTRL_R = 0;         // disable SysTick during setup
  NVIC_ST_RELOAD_R = period-1;// reload value
  NVIC_ST_CURRENT_R = 0;      // any write to current clears it
  NVIC_SYS_PRI3_R = (NVIC_SYS_PRI3_R&0x00FFFFFF)|0x40000000; // priority 2

  SYSCTL_RCGC2_R |= 0x00000010;   // 1) activate clock for Port E
  delay = SYSCTL_RCGC2_R;         //    allow time for clock to stabilize

	// enable SysTick with core clock and interrupts
  NVIC_ST_CTRL_R = 0x07;
  EnableInterrupts();
}

// ADC: executes every 25 ms, collects a sample, converts and stores in mailbox
void SysTick_Handler(void){ 
	  volatile unsigned long delay;
	 // unsigned long Distance = ADC0_In();// initialize profiling on PF1 (optional)
		delay = SYSCTL_RCGC2_R;         //    allow time for clock to stabilize
	  ADCdata = ADC0_In(); // Gets Analog input
	  Flag=1; // Data ready!
	
}

void lcdwrite(void);
unsigned long ADCdata;


		unsigned long LeerDato(void) {
			
	  unsigned long result,sum=0;
		int i;
    for(i=0;i<128;i++){   // 2048 means result=sum>>11
		  ADCdata = ADC0_In(); // Get Analog input from acquisition function
      sum += ADCdata; // Adds value from Analog input
      }
      result=sum>>7; //As this for iteration gets 16 values we have to move 4 bits result to the right
			               // If we want to add 128 values for the mean, we have to move result 7 bits to the right
			return result;
		}

int main(void){
  volatile unsigned long delay;
  //TExaS_Init(ADC0_AIN1_PIN_PE2, SSI0_Real_Nokia5110_Scope);

	// initialize ADC0, channel 1, sequencer 3
  Nokia5110_Init();// initialize Nokia5110 LCD (optional)
	
	// As Nokia display is the slowest device, I optimized it at top speed of SysTick for it
	// Former initialize SysTick = 2000000 for 40 Hz interrupts
	  SysTick_Init(1500); // initialize SysTick = 1500 means 18,8 miliseconds speed of sampling = 53,3 kHz interrupts (the faster before system does not responds)
  // initialize profiling on PF1 (optional)
  ADC0_Init(); 
	delay = SYSCTL_RCGC2_R;         //    allow time for clock to stabilize

  EnableInterrupts();
	
  PLL_Init(); // bus clock at 3.2 MHz
	
//  PWM0Dual_Init for initialisation
//  PWM0Dual_Init(60000,50,50);      // initialize PWM1-0, 0.8333 Hz, 1 RPM (1 Hz)
//  PWM0Dual_Init(30000,50,50);      // initialize PWM1-0, 0.8333 Hz, 2 RPM
//  PWM0Dual_Init(15000,50,50);      // initialize PWM1-0, 0.8333 Hz, 4 RPM
  
	PWM0Dual_Init(1250,50,50);      // initialize PWM1-0, 50 Hz,  20 ms for servo control
	PWM1Dual_Init(1250,50,50);      // initialize PWM1-0, 50 Hz,  20 ms for servo control

 while(1){ 
	//	unsigned long Value;
  // read mailbox
  //output to Nokia5110 LCD (optional)
    if(Flag==1){
	  
		//ADCdata = ADC0_In();
		//Distance=(((ADCdata)*3000)/2047);
		
    // Corrected formulae Distance=((((LeerDato())*3000)/2047)*0.97)+180;
		// Corrected OK execepting zero error Distance=((((LeerDato())*3000)/2047)*0.969)+186.3911289;
		/*Distance=
			(((LeerDato())*3000)/2048)*1.003
			+((((LeerDato())*3000)/2048)*(((LeerDato())*3000)/2048))*0.00000001
			+(5/(0.5+(((LeerDato())*3000)/2048)));
		*/
		//Distance=((((LeerDato())*3000)/2047))*0.969-10;
		Distance=((((LeerDato())*3000)/2047))*0.965-10; // Calibration for my 6 cm potentiometer

					
		Nokia5110_Clear();
    Nokia5110_SetCursor(0, 0);
    //Distance = Convert(ADCdata);
		Nokia5110_OutString("Embedded Systems"); 
		Nokia5110_SetCursor(0, 1);			
		Nokia5110_OutString("@Franciscodr"); 
		Nokia5110_SetCursor(0, 3);
		Nokia5110_OutString("Position:"); 
		Nokia5110_SetCursor(0, 4);					
    UART_ConvertDistance(Distance); // from Lab 11
		
    Nokia5110_OutString(String); // Prints out at display   
    
		/*
 Main PWM function call
 PWM0Dual_Period (dcpwm0,dcpwm1) 
 where dcpwm0 and dcpwm1 go from 0 to 65535
 change of dcpwm1 is not recomended as affects main PWM output
 so keep dcpwm1 at zero value
	
This values are based on real test and calculations derived from them
for Duty Cycle at "dcpwm0" value at "PWM0Dual_Period" PWM output function

Position 
of servo in     Calculated 
Degrees	        positions
   0	  ---------- 2100  (or +90 as said above)
   45   ---------- 3466
   90   ---------- 4833  (or  0 as said above)
   135  ---------- 6199
   180	---------- 7565  (or -90 as said above)
	 
1 degree ----------	 30

As dcpwm0 and dcpwm1 are unsigned short variables 65535 is the maximum value for them
But for PWM servo motor control change only dcpwm0 between values shown just above
		*/
/* Calculations for adjustment between Distance measured with potentiomenter
and position set by PWM

output = servo = from 2100 to 7565
function(potentiometer) = 2100 + K*x
for a position of potentiomenter of x = 0
f = 2100

for a position of potentiomenter of x = 5.780
f = 7565

K = (7565 - 2100)/5.780
K = 945,5017301038062 (if position is in thousands)
K = 0,9455017301038062 (if position is in units)
*/
		
  PWM0Dual_Period(((Distance*0.9455)+2100),0);           // 4833 input = 0 (middle) position for servo motor
	PWM1Dual_Period(((Distance*0.9455)+2100),0);           // 4833 input = 0 (middle) position for servo motor
//  PWM0Dual_Period(2100,0);           // 2100 input = +90  relative position for servo motor
//  PWM0Dual_Period(7565,0);           // 7565 input = -90  relative position for servo motor

// From old documentation:
//  PWM0Dual_Period(2400);           // 20.83 Hz, 25 RPM
//  PWM0Dual_Period(667);            // 75.00 Hz, 90 RPM
//  PWM0Dual_Period(600);            // 83.33 Hz, 100 RPM
// End old documentation
    WaitForInterrupt();	
		}
    Flag=0; // Flag reset to send the signal that last measure has been diplayed
  }

}

//-----------------------UART_ConvertDistance-----------------------
// Converts a 32-bit distance into an ASCII string
// Input: 32-bit number to be converted (resolution 0.001cm)
// Output: store the conversion in global variable String[10]
// Fixed format 1 digit, point, 3 digits, space, units, null termination
// Examples
//    4 to "0.004 cm"  
//   31 to "0.031 cm" 
//  102 to "0.102 cm" 
// 2210 to "2.210 cm"
//10000 to "*.*** cm"  any value larger than 9999 converted to "*.*** cm"
void UART_ConvertDistance(unsigned long n){
// as part of Lab 11 you implemented this function
// as part of Lab 11 implement this function
  if(n < 10) {
		String[0] = '0';
		String[2] = '0';
		String[3] = '0';
		String[4] = n + '0';
	} else if (n < 100) {
		String[0] = '0';
		String[2] = '0';
		String[3] = n/10 + '0';
		String[4] = (n%10) + '0';
	} else if (n < 1000) {
		String[0] = '0';
		String[2] = n/100 + '0';
		String[3] = (n%100)/10 + '0';
		String[4] = (n%10) + '0';
	} else if (n < 10000) {
		String[0] = n/1000 + '0';
		String[2] = (n%1000)/100 + '0';
		String[3] = (n%100)/10 + '0';
		String[4] = (n%10) + '0';
	} else {
		String[0] = '*';
		String[2] = '*';
		String[3] = '*';
		String[4] = '*';
	}
	String[1] = '.';
	String[5] = ' ';
	String[6] = 'c';
	String[7] = 'm';
	String[8] = '\0';
}
